/*
 * Base class for overlay widgets.
 * It is not supposed to go into any kind of layout.
 * The widget will position itself depending on it's container size
 */

#include "overlaywidget.h"

OverlayWidget::OverlayWidget(ContainerWidget *parent) : QWidget(parent) {
    this->setAccessibleName("OverlayWidget");
    connect(parent, SIGNAL(resized(QSize)), this, SLOT(setContainerSize(QSize)));
    hide();
}

QSize OverlayWidget::containerSize() {
    return container;
}

void OverlayWidget::setContainerSize(QSize container) {
    this->container = container;
    recalculateGeometry();
}

void OverlayWidget::paintEvent(QPaintEvent *event) {
    QStyleOption opt;
    opt.init(this);
    QPainter p(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
}
